@@ -644,13 +644,11 @@ class PageObject(DictionaryObject):
         stream = ContentStream(stream, pdf)
         for operands, _operator in stream.operations:
             if isinstance(operands, list):
-                for i in range(len(operands)):
-                    op = operands[i]
+                for i, op in enumerate(operands):
                     if isinstance(op, NameObject):
                         operands[i] = rename.get(op, op)
             elif isinstance(operands, dict):
-                for i in operands:
-                    op = operands[i]
+                for i, op in operands.items():
                     if isinstance(op, NameObject):
                         operands[i] = rename.get(op, op)
             else:
@@ -703,7 +701,11 @@ class PageObject(DictionaryObject):
             ``/Contents`` is optional, as described in PDF Reference  7.7.3.3
         """
         if PG.CONTENTS in self:
-            return self[PG.CONTENTS].get_object()  # type: ignore
+            try:
+                pdf = cast(IndirectObject, self.indirect_reference).pdf
+            except AttributeError:
+                pdf = None
+            return ContentStream(self[PG.CONTENTS].get_object(), pdf)
         else:
             return None
 
@@ -821,7 +823,6 @@ class PageObject(DictionaryObject):
 
         page2content = page2.get_contents()
         if page2content is not None:
-            page2content = ContentStream(page2content, self.pdf)
             rect = getattr(page2, MERGE_CROP_BOX)
             page2content.operations.insert(
                 0,
@@ -957,7 +958,6 @@ class PageObject(DictionaryObject):
 
         page2content = page2.get_contents()
         if page2content is not None:
-            page2content = ContentStream(page2content, self.pdf)
             rect = getattr(page2, MERGE_CROP_BOX)
             page2content.operations.insert(
                 0,
@@ -1493,12 +1493,7 @@ class PageObject(DictionaryObject):
         """
         content = self.get_contents()
         if content is not None:
-            content_obj: Any
-            if not isinstance(content, ContentStream):
-                content_obj = ContentStream(content, self.pdf)
-            else:
-                content_obj = content
-            content_obj = content_obj.flate_encode()
+            content_obj = content.flate_encode()
             try:
                 content.indirect_reference.pdf._objects[  # type: ignore
                     content.indirect_reference.idnum - 1  # type: ignore
